@@ -68,6 +68,67 @@
  * a #ClutterAnimation instance and animate an actor between its current
  * state and the specified final state.
  *
+ * <refsect2 id="clutter-AnimationMode-Script">
+ *   <title>Defining ClutterAnimationMode inside ClutterScript</title>
+ *   <para>When defining a #ClutterAnimation inside a ClutterScript
+ *   file or string the #ClutterAnimation:mode can be defined either
+ *   using the #ClutterAnimationMode enumeration values through their
+ *   "nick" (the short string used inside #GEnumValue), their numeric
+ *   id, or using the following strings:</para>
+ *   <variablelist>
+ *     <varlistentry>
+ *       <term>easeInQuad, easeOutQuad, easeInOutQuad</term>
+ *       <listitem><para>Corresponding to the quadratic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInCubic, easeOutCubic, easeInOutCubic</term>
+ *       <listitem><para>Corresponding to the cubic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInQuart, easeOutQuart, easeInOutQuart</term>
+ *       <listitem><para>Corresponding to the quartic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInQuint, easeOutQuint, easeInOutQuint</term>
+ *       <listitem><para>Corresponding to the quintic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInSine, easeOutSine, easeInOutSine</term>
+ *       <listitem><para>Corresponding to the sine easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInExpo, easeOutExpo, easeInOutExpo</term>
+ *       <listitem><para>Corresponding to the exponential easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInCirc, easeOutCirc, easeInOutCirc</term>
+ *       <listitem><para>Corresponding to the circular easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInElastic, easeOutElastic, easeInOutElastic</term>
+ *       <listitem><para>Corresponding to the overshooting elastic
+ *       easing modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInBack, easeOutBack, easeInOutBack</term>
+ *       <listitem><para>Corresponding to the overshooting cubic
+ *       easing modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInBounce, easeOutBounce, easeInOutBounce</term>
+ *       <listitem><para>Corresponding to the bouncing easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *   </variablelist>
+ * </refsect2>
+ *
  * #ClutterAnimation is available since Clutter 1.0
  */
 
@@ -85,6 +146,8 @@
 #include "clutter-enum-types.h"
 #include "clutter-interval.h"
 #include "clutter-private.h"
+#include "clutter-scriptable.h"
+#include "clutter-script-private.h"
 
 enum
 {
@@ -125,7 +188,25 @@ static guint animation_signals[LAST_SIGNAL] = { 0, };
 
 static GQuark quark_object_animation = 0;
 
-G_DEFINE_TYPE (ClutterAnimation, clutter_animation, G_TYPE_OBJECT);
+static void clutter_scriptable_init (ClutterScriptableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (ClutterAnimation, clutter_animation, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                clutter_scriptable_init));
+
+static void
+on_actor_dispose (gpointer  user_data,
+                  GObject  *actor_pointer)
+{
+  ClutterAnimation *self = user_data;
+
+  if (self->priv->object == actor_pointer)
+    {
+      CLUTTER_NOTE (ANIMATION, "Object [%p] was unref'd", actor_pointer);
+      g_object_unref (self);
+    }
+}
+
 
 static void
 clutter_animation_real_completed (ClutterAnimation *self)
@@ -141,6 +222,7 @@ clutter_animation_real_completed (ClutterAnimation *self)
   direction = clutter_timeline_get_direction (timeline);
 
   /* explicitly set the final state of the animation */
+  CLUTTER_NOTE (ANIMATION, "Set final state on object [%p]", priv->object);
   g_hash_table_iter_init (&iter, priv->properties);
   while (g_hash_table_iter_next (&iter, &key, &value))
     {
@@ -167,7 +249,14 @@ clutter_animation_real_completed (ClutterAnimation *self)
   animation = g_object_get_qdata (priv->object, quark_object_animation);
   if (animation == self)
     {
+      CLUTTER_NOTE (ANIMATION, "Unsetting animation for actor [%p]",
+                    priv->object);
+
       g_object_set_qdata (priv->object, quark_object_animation, NULL);
+      g_object_weak_unref (priv->object, on_actor_dispose, self);
+
+      CLUTTER_NOTE (ANIMATION, "Releasing the reference Animation [%p]",
+                    animation);
       g_object_unref (animation);
     }
 }
@@ -177,7 +266,9 @@ clutter_animation_finalize (GObject *gobject)
 {
   ClutterAnimationPrivate *priv = CLUTTER_ANIMATION (gobject)->priv;
 
-  CLUTTER_NOTE (ANIMATION, "Destroying properties hash table");
+  CLUTTER_NOTE (ANIMATION,
+                "Destroying properties table for Animation [%p]",
+                gobject);
   g_hash_table_destroy (priv->properties);
 
   G_OBJECT_CLASS (clutter_animation_parent_class)->finalize (gobject);
@@ -303,6 +394,34 @@ clutter_animation_get_property (GObject    *gobject,
     }
 }
 
+static gboolean
+clutter_animation_parse_custom_node (ClutterScriptable *scriptable,
+                                     ClutterScript     *script,
+                                     GValue            *value,
+                                     const gchar       *name,
+                                     JsonNode          *node)
+{
+  if (strncmp (name, "mode", 4) == 0)
+    {
+      gulong mode;
+
+      mode = clutter_script_resolve_animation_mode (node);
+
+      g_value_init (value, G_TYPE_ULONG);
+      g_value_set_ulong (value, mode);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static void
+clutter_scriptable_init (ClutterScriptableIface *iface)
+{
+  iface->parse_custom_node = clutter_animation_parse_custom_node;
+}
+
 static void
 clutter_animation_class_init (ClutterAnimationClass *klass)
 {
@@ -322,7 +441,7 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
   gobject_class->finalize = clutter_animation_finalize;
 
   /**
-   * ClutterAnimation:objct:
+   * ClutterAnimation:object:
    *
    * The #GObject to which the animation applies.
    *
@@ -507,6 +626,7 @@ clutter_animation_validate_bind (ClutterAnimation *animation,
   ClutterAnimationPrivate *priv;
   GObjectClass *klass;
   GParamSpec *pspec;
+  GType pspec_type;
 
   priv = animation->priv;
 
@@ -545,17 +665,20 @@ clutter_animation_validate_bind (ClutterAnimation *animation,
       return NULL;
     }
 
-  if (!g_value_type_compatible (G_PARAM_SPEC_VALUE_TYPE (pspec),
-                                argtype))
+  pspec_type = G_PARAM_SPEC_VALUE_TYPE (pspec);
+
+  if (!g_value_type_compatible (argtype, pspec_type) ||
+      !g_value_type_transformable (argtype, pspec_type))
     {
       g_warning ("Cannot bind property '%s': the interval value of "
                  "type '%s' is not compatible with the property value "
                  "of type '%s'",
                  property_name,
                  g_type_name (argtype),
-                 g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
+                 g_type_name (pspec_type));
       return NULL;
     }
+
   return pspec;
 }
 
@@ -632,8 +755,7 @@ clutter_animation_bind (ClutterAnimation *animation,
   priv = animation->priv;
 
   type = G_VALUE_TYPE (final);
-  pspec = clutter_animation_validate_bind (animation, property_name,
-                                           type);
+  pspec = clutter_animation_validate_bind (animation, property_name, type);
   if (pspec == NULL)
     return NULL;
 
@@ -725,6 +847,7 @@ clutter_animation_update_interval (ClutterAnimation *animation,
   ClutterAnimationPrivate *priv;
   GObjectClass *klass;
   GParamSpec *pspec;
+  GType pspec_type, int_type;
 
   g_return_if_fail (CLUTTER_IS_ANIMATION (animation));
   g_return_if_fail (property_name != NULL);
@@ -751,15 +874,18 @@ clutter_animation_update_interval (ClutterAnimation *animation,
       return;
     }
 
-  if (!g_value_type_compatible (G_PARAM_SPEC_VALUE_TYPE (pspec),
-                                clutter_interval_get_value_type (interval)))
+  pspec_type = G_PARAM_SPEC_VALUE_TYPE (pspec);
+  int_type = clutter_interval_get_value_type (interval);
+
+  if (!g_value_type_compatible (int_type, pspec_type) ||
+      !g_value_type_transformable (int_type, pspec_type))
     {
       g_warning ("Cannot update property '%s': the interval value of "
                  "type '%s' is not compatible with the property value "
                  "of type '%s'",
                  property_name,
-                 g_type_name (clutter_interval_get_value_type (interval)),
-                 g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
+                 g_type_name (int_type),
+                 g_type_name (pspec_type));
       return;
     }
 
@@ -785,6 +911,7 @@ clutter_animation_update (ClutterAnimation *animation,
 {
   ClutterAnimationPrivate *priv;
   ClutterInterval *interval;
+  GType int_type;
 
   g_return_val_if_fail (CLUTTER_IS_ANIMATION (animation), NULL);
   g_return_val_if_fail (property_name != NULL, NULL);
@@ -802,14 +929,16 @@ clutter_animation_update (ClutterAnimation *animation,
       return NULL;
     }
 
-  if (!g_value_type_compatible (G_VALUE_TYPE (final),
-                                clutter_interval_get_value_type (interval)))
+  int_type = clutter_interval_get_value_type (interval);
+
+  if (!g_value_type_compatible (G_VALUE_TYPE (final), int_type) ||
+      !g_value_type_transformable (G_VALUE_TYPE (final), int_type))
     {
       g_warning ("Cannot update property '%s': the interval value of "
                  "type '%s' is not compatible with the property value "
                  "of type '%s'",
                  property_name,
-                 g_type_name (clutter_interval_get_value_type (interval)),
+                 g_type_name (int_type),
                  g_type_name (G_VALUE_TYPE (final)));
       return NULL;
     }
@@ -1100,6 +1229,8 @@ clutter_animation_set_mode (ClutterAnimation *animation,
   g_object_freeze_notify (G_OBJECT (animation));
 
   alpha = clutter_animation_get_alpha_internal (animation);
+  g_assert (CLUTTER_IS_ALPHA (alpha));
+
   clutter_alpha_set_mode (alpha, mode);
 
   g_object_notify (G_OBJECT (animation), "mode");
@@ -1153,6 +1284,8 @@ clutter_animation_set_duration (ClutterAnimation *animation,
   g_object_freeze_notify (G_OBJECT (animation));
 
   timeline = clutter_animation_get_timeline_internal (animation);
+  g_assert (CLUTTER_IS_TIMELINE (timeline));
+
   clutter_timeline_set_duration (timeline, msecs);
   clutter_timeline_rewind (timeline);
 
@@ -1513,35 +1646,37 @@ clutter_animation_setup_property (ClutterAnimation *animation,
    */
   if (!g_type_is_a (G_VALUE_TYPE (value), G_VALUE_TYPE (&real_value)))
     {
-      if (!g_value_type_compatible (G_VALUE_TYPE (value),
-                                    G_VALUE_TYPE (&real_value)) &&
-          !g_value_type_compatible (G_VALUE_TYPE (&real_value),
-                                    G_VALUE_TYPE (value)))
+      /* are these two types compatible (can be directly copied)? */
+      if (g_value_type_compatible (G_VALUE_TYPE (value),
+                                   G_VALUE_TYPE (&real_value)))
         {
-          g_warning ("%s: Unable to convert from %s to %s for "
-                     "the property '%s' of object %s",
-                     G_STRLOC,
-                     g_type_name (G_VALUE_TYPE (value)),
-                     g_type_name (G_VALUE_TYPE (&real_value)),
-                     property_name,
-                     G_OBJECT_TYPE_NAME (priv->object));
-          g_value_unset (&real_value);
-          return;
+          g_value_copy (value, &real_value);
+          goto done;
         }
 
-      if (!g_value_transform (value, &real_value))
+      /* are these two type transformable? */
+      if (g_value_type_transformable (G_VALUE_TYPE (value),
+                                      G_VALUE_TYPE (&real_value)))
         {
-          g_warning ("%s: Unable to transform from %s to %s",
-                     G_STRLOC,
-                     g_type_name (G_VALUE_TYPE (value)),
-                     g_type_name (G_VALUE_TYPE (&real_value)));
-          g_value_unset (&real_value);
-          return;
+          if (g_value_transform (value, &real_value))
+            goto done;
         }
+
+      /* if not compatible and not transformable then we can't do much */
+      g_warning ("%s: Unable to convert from %s to %s for "
+                 "the property '%s' of object %s",
+                 G_STRLOC,
+                 g_type_name (G_VALUE_TYPE (value)),
+                 g_type_name (G_VALUE_TYPE (&real_value)),
+                 property_name,
+                 G_OBJECT_TYPE_NAME (priv->object));
+      g_value_unset (&real_value);
+      return;
     }
   else
     g_value_copy (value, &real_value);
 
+done:
   /* create an interval and bind it to the property, in case
    * it's not a fixed property, otherwise just set it
    */
@@ -1731,6 +1866,7 @@ animation_create_for_actor (ClutterActor *actor)
       animation = clutter_animation_new ();
       clutter_animation_set_object (animation, object);
       g_object_set_qdata (object, quark_object_animation, animation);
+      g_object_weak_ref (object, on_actor_dispose, animation);
 
       CLUTTER_NOTE (ANIMATION,
                     "Created new Animation [%p] for actor [%p]",
