@@ -54,6 +54,32 @@ clutter_allocation_flags_get_type (void)
   return g_enum_type_id__volatile;
 }
 
+/* enumerations from "./clutter-animator.h" */
+#include "./clutter-animator.h"
+
+GType
+clutter_interpolation_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_INTERPOLATION_LINEAR, "CLUTTER_INTERPOLATION_LINEAR", "linear" },
+        { CLUTTER_INTERPOLATION_CUBIC, "CLUTTER_INTERPOLATION_CUBIC", "cubic" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterInterpolation"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
 /* enumerations from "./clutter-bin-layout.h" */
 #include "./clutter-bin-layout.h"
 
@@ -253,30 +279,6 @@ clutter_stage_state_get_type (void)
 
   return g_enum_type_id__volatile;
 }
-GType
-clutter_input_device_type_get_type (void)
-{
-  static volatile gsize g_enum_type_id__volatile = 0;
-
-  if (g_once_init_enter (&g_enum_type_id__volatile))
-    {
-      static const GEnumValue values[] = {
-        { CLUTTER_POINTER_DEVICE, "CLUTTER_POINTER_DEVICE", "pointer-device" },
-        { CLUTTER_KEYBOARD_DEVICE, "CLUTTER_KEYBOARD_DEVICE", "keyboard-device" },
-        { CLUTTER_EXTENSION_DEVICE, "CLUTTER_EXTENSION_DEVICE", "extension-device" },
-        { CLUTTER_N_DEVICE_TYPES, "CLUTTER_N_DEVICE_TYPES", "n-device-types" },
-        { 0, NULL, NULL }
-      };
-      GType g_enum_type_id;
-
-      g_enum_type_id =
-        g_enum_register_static (g_intern_static_string ("ClutterInputDeviceType"), values);
-
-      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
-    }
-
-  return g_enum_type_id__volatile;
-}
 
 /* enumerations from "./clutter-feature.h" */
 #include "./clutter-feature.h"
@@ -338,6 +340,34 @@ clutter_flow_orientation_get_type (void)
   return g_enum_type_id__volatile;
 }
 
+/* enumerations from "./clutter-input-device.h" */
+#include "./clutter-input-device.h"
+
+GType
+clutter_input_device_type_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_POINTER_DEVICE, "CLUTTER_POINTER_DEVICE", "pointer-device" },
+        { CLUTTER_KEYBOARD_DEVICE, "CLUTTER_KEYBOARD_DEVICE", "keyboard-device" },
+        { CLUTTER_EXTENSION_DEVICE, "CLUTTER_EXTENSION_DEVICE", "extension-device" },
+        { CLUTTER_N_DEVICE_TYPES, "CLUTTER_N_DEVICE_TYPES", "n-device-types" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterInputDeviceType"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
 /* enumerations from "./clutter-main.h" */
 #include "./clutter-main.h"
 
