@@ -38,6 +38,7 @@
 #include "clutter-master-clock.h"
 #include "clutter-debug.h"
 #include "clutter-private.h"
+#include "clutter-profile.h"
 
 #define CLUTTER_MASTER_CLOCK_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST ((klass), CLUTTER_TYPE_MASTER_CLOCK, ClutterMasterClockClass))
 #define CLUTTER_IS_MASTER_CLOCK_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE ((klass), CLUTTER_TYPE_MASTER_CLOCK))
@@ -178,6 +179,17 @@ master_clock_next_frame_delay (ClutterMasterClock *master_clock)
   g_source_get_current_time (master_clock->source, &now);
 
   next = master_clock->prev_tick;
+
+  /* If time has gone backwards then there's no way of knowing how
+     long we should wait so let's just dispatch immediately */
+  if (now.tv_sec < next.tv_sec ||
+      (now.tv_sec == next.tv_sec && now.tv_usec <= next.tv_usec))
+    {
+      CLUTTER_NOTE (SCHEDULER, "Time has gone backwards");
+
+      return 0;
+    }
+
   g_time_val_add (&next, 1000000L / (gulong) clutter_get_default_frame_rate ());
 
   if (next.tv_sec < now.tv_sec ||
@@ -262,6 +274,19 @@ clutter_clock_dispatch (GSource     *source,
   ClutterStageManager *stage_manager = clutter_stage_manager_get_default ();
   GSList *stages, *l;
 
+  CLUTTER_STATIC_TIMER (master_dispatch_timer,
+                        "Mainloop",
+                        "Master Clock",
+                        "Master clock dispatch",
+                        0);
+  CLUTTER_STATIC_TIMER (master_event_process,
+                        "Master Clock",
+                        "Event Processing",
+                        "The time spent processing events on all stages",
+                        0);
+
+  CLUTTER_TIMER_START (_clutter_uprof_context, master_dispatch_timer);
+
   CLUTTER_NOTE (SCHEDULER, "Master clock [tick]");
 
   clutter_threads_enter ();
@@ -274,16 +299,20 @@ clutter_clock_dispatch (GSource     *source,
    * event handling
    */
   stages = clutter_stage_manager_list_stages (stage_manager);
-  g_slist_foreach (stages, (GFunc)g_object_ref, NULL);
+  g_slist_foreach (stages, (GFunc) g_object_ref, NULL);
+
+  CLUTTER_TIMER_START (_clutter_uprof_context, master_event_process);
 
   master_clock->updated_stages = FALSE;
 
-  /* Process queued events
-   */
+  /* Process queued events */
   for (l = stages; l != NULL; l = l->next)
     _clutter_stage_process_queued_events (l->data);
 
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, master_event_process);
+
   _clutter_master_clock_advance (master_clock);
+
   _clutter_run_repaint_functions ();
 
   /* Update any stage that needs redraw/relayout after the clock
@@ -292,13 +321,15 @@ clutter_clock_dispatch (GSource     *source,
   for (l = stages; l != NULL; l = l->next)
     master_clock->updated_stages |= _clutter_stage_do_update (l->data);
 
-  g_slist_foreach (stages, (GFunc)g_object_unref, NULL);
+  g_slist_foreach (stages, (GFunc) g_object_unref, NULL);
   g_slist_free (stages);
 
   master_clock->prev_tick = master_clock->cur_tick;
 
   clutter_threads_leave ();
 
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, master_dispatch_timer);
+
   return TRUE;
 }
 
@@ -425,23 +456,48 @@ _clutter_master_clock_start_running (ClutterMasterClock *master_clock)
 void
 _clutter_master_clock_advance (ClutterMasterClock *master_clock)
 {
-  GSList *l, *next;
+  GSList *timelines, *l;
+
+  CLUTTER_STATIC_TIMER (master_timeline_advance,
+                        "Master Clock",
+                        "Timelines Advancement",
+                        "The time spent advancing all timelines",
+                        0);
 
   g_return_if_fail (CLUTTER_IS_MASTER_CLOCK (master_clock));
 
+  CLUTTER_TIMER_START (_clutter_uprof_context, master_timeline_advance);
+
   /* we protect ourselves from timelines being removed during
-   * the advancement by other timelines
+   * the advancement by other timelines by copying the list of
+   * timelines, taking a reference on them, iterating over the
+   * copied list and then releasing the reference.
+   *
+   * we cannot simply take a reference on the timelines and still
+   * use the list held by the master clock because the do_tick()
+   * might result in the creation of a new timeline, which gets
+   * added at the end of the list with no reference increase and
+   * thus gets disposed at the end of the iteration.
+   *
+   * this implies that a newly added timeline will not be advanced
+   * by this clock iteration, which is perfectly fine since we're
+   * in its first cycle.
+   *
+   * we also cannot steal the master clock timelines list because
+   * a timeline might be removed as the direct result of do_tick()
+   * and remove_timeline() would not find the timeline, failing
+   * and leaving a dangling pointer behind.
    */
-  g_slist_foreach (master_clock->timelines, (GFunc) g_object_ref, NULL);
+  timelines = g_slist_copy (master_clock->timelines);
+  g_slist_foreach (timelines, (GFunc) g_object_ref, NULL);
 
-  for (l = master_clock->timelines; l != NULL; l = next)
-    {
-      next = l->next;
+  for (l = timelines; l != NULL; l = l->next)
+    clutter_timeline_do_tick (l->data, &master_clock->cur_tick);
 
-      clutter_timeline_do_tick (l->data, &master_clock->cur_tick);
-    }
+  g_slist_foreach (timelines, (GFunc) g_object_unref, NULL);
+  g_slist_free (timelines);
 
-  g_slist_foreach (master_clock->timelines, (GFunc) g_object_unref, NULL);
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, master_timeline_advance);
 }
 
 /**
