@@ -29,6 +29,7 @@
 #define __COGL_TEXTURE_H__
 
 #include <cogl/cogl-types.h>
+#include <cogl/cogl-defines.h>
 
 G_BEGIN_DECLS
 
@@ -316,6 +317,93 @@ gboolean        cogl_texture_set_region       (CoglHandle          handle,
                                                guint               rowstride,
                                                const guchar       *data);
 
+/**
+ * cogl_texture_new_from_sub_texture:
+ * @full_texture: a #CoglHandle to an existing texture
+ * @sub_x: X coordinate of the top-left of the subregion
+ * @sub_y: Y coordinate of the top-left of the subregion
+ * @sub_width: Width in pixels of the subregion
+ * @sub_height: Height in pixels of the subregion
+ *
+ * Creates a new texture which represents a subregion of another
+ * texture. The GL resources will be shared so that no new texture
+ * data is actually allocated.
+ *
+ * Sub textures have undefined behaviour texture coordinates outside
+ * of the range [0,1] are used. They also do not work with
+ * CoglVertexBuffers.
+ *
+ * Return value: a #CoglHandle to the new texture.
+ *
+ * Since: 1.2
+ */
+CoglHandle      cogl_texture_new_from_sub_texture
+                                              (CoglHandle full_texture,
+                                               gint       sub_x,
+                                               gint       sub_y,
+                                               gint       sub_width,
+                                               gint       sub_height);
+
+#if defined (COGL_ENABLE_EXPERIMENTAL_API)
+
+/**
+ * cogl_texture_new_from_buffer:
+ * @buffer: the #CoglHandle of a pixel buffer
+ * @width: width of texture in pixels or 0
+ * @height: height of texture in pixels or 0
+ * @flags: optional flags for the texture, or %COGL_TEXTURE_NONE
+ * @format: the #CoglPixelFormat the buffer is stored in in RAM
+ * @internal_format: the #CoglPixelFormat that will be used for storing
+ *    the buffer on the GPU. If %COGL_PIXEL_FORMAT_ANY is given then a
+ *    premultiplied format similar to the format of the source data will
+ *    be used. The default blending equations of Cogl expect premultiplied
+ *    color data; the main use of passing a non-premultiplied format here
+ *    is if you have non-premultiplied source data and are going to adjust
+ *    the blend mode (see cogl_material_set_blend()) or use the data for
+ *    something other than straight blending
+ * @rowstride: the memory offset in bytes between the starts of
+ *    scanlines in @data. If 0 is given the row stride will be deduced from
+ *    @width and @format or the stride given by cogl_pixel_buffer_new_for_size()
+ * @offset: offset in bytes in @buffer from where the texture data starts
+ *
+ * Creates a new texture using the buffer specified by @handle. If the buffer
+ * has been created using cogl_pixel_buffer_new_for_size() it's possible to omit
+ * the height and width values already specified at creation time.
+ *
+ * Return value: a #CoglHandle to the new texture or %COGL_INVALID_HANDLE on
+ *    failure
+ *
+ * Since: 1.2
+ * Stability: Unstable
+ */
+CoglHandle      cogl_texture_new_from_buffer  (CoglHandle       buffer,
+                                               guint            width,
+                                               guint            height,
+                                               CoglTextureFlags flags,
+                                               CoglPixelFormat  format,
+                                               CoglPixelFormat  internal_format,
+                                               guint            rowstride,
+                                               guint            offset);
+
+/* the function above is experimental, the actual symbol is suffixed by _EXP so
+ * we can ensure ABI compatibility and leave the cogl_buffer namespace free for
+ * future use. A bunch of defines translates the symbols documented above into
+ * the real symbols */
+
+CoglHandle      cogl_texture_new_from_buffer_EXP
+                                              (CoglHandle       buffer,
+                                               guint            width,
+                                               guint            height,
+                                               CoglTextureFlags flags,
+                                               CoglPixelFormat  format,
+                                               CoglPixelFormat  internal_format,
+                                               guint            rowstride,
+                                               guint            offset);
+
+#define cogl_texture_new_from_buffer cogl_texture_new_from_buffer_EXP
+
+#endif
+
 #ifndef COGL_DISABLE_DEPRECATED
 
 /**
@@ -460,15 +548,16 @@ void cogl_rectangles (const float *verts,
  * All of the texture coordinates must be in the range [0,1] and repeating the
  * texture is not supported.
  *
- * Because of the way this function is implemented it will currently only work
- * if either the texture is not sliced or the backend is not OpenGL ES and the
- * minifying and magnifying functions are both set to CGL_NEAREST.
+ * Because of the way this function is implemented it will currently
+ * only work if either the texture is not sliced or the backend is not
+ * OpenGL ES and the minifying and magnifying functions are both set
+ * to COGL_MATERIAL_FILTER_NEAREST.
  *
  * Since: 1.0
  */
-void cogl_polygon (CoglTextureVertex  *vertices,
-                   guint               n_vertices,
-                   gboolean            use_color);
+void cogl_polygon (const CoglTextureVertex  *vertices,
+                   guint                     n_vertices,
+                   gboolean                  use_color);
 
 G_END_DECLS
 
