@@ -42,6 +42,7 @@
 #include "clutter-backend-glx.h"
 #include "clutter-stage-glx.h"
 #include "clutter-glx.h"
+#include "clutter-profile.h"
 
 #include "../clutter-event.h"
 #include "../clutter-main.h"
@@ -51,6 +52,7 @@
 
 #include "cogl/cogl.h"
 
+
 G_DEFINE_TYPE (ClutterBackendGLX, clutter_backend_glx, CLUTTER_TYPE_BACKEND_X11);
 
 /* singleton object */
@@ -185,10 +187,17 @@ clutter_backend_glx_dispose (GObject *gobject)
 
   if (backend_glx->gl_context)
     {
+      glXMakeContextCurrent (backend_x11->xdpy, None, None, NULL);
       glXDestroyContext (backend_x11->xdpy, backend_glx->gl_context);
       backend_glx->gl_context = None;
     }
 
+  if (backend_glx->dummy_xwin)
+    {
+      XDestroyWindow (backend_x11->xdpy, backend_glx->dummy_xwin);
+      backend_glx->dummy_xwin = None;
+    }
+
   G_OBJECT_CLASS (clutter_backend_glx_parent_class)->dispose (gobject);
 }
 
@@ -228,24 +237,23 @@ check_vblank_env (const char *name)
 static ClutterFeatureFlags
 clutter_backend_glx_get_features (ClutterBackend *backend)
 {
-  ClutterBackendGLX  *backend_glx = CLUTTER_BACKEND_GLX (backend);
-  const gchar        *glx_extensions = NULL;
+  ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (backend);
+  const gchar *glx_extensions = NULL;
   ClutterFeatureFlags flags;
 
   flags = clutter_backend_x11_get_features (backend);
   flags |= CLUTTER_FEATURE_STAGE_MULTIPLE;
 
-  /* this will make sure that the GL context exists and
-   * it's bound to a drawable
-   */
+  /* this will make sure that the GL context exists */
   g_assert (backend_glx->gl_context != None);
   g_assert (glXGetCurrentDrawable () != None);
 
-  CLUTTER_NOTE (BACKEND, "Checking features\n"
-                "GL_VENDOR: %s\n"
-                "GL_RENDERER: %s\n"
-                "GL_VERSION: %s\n"
-                "GL_EXTENSIONS: %s\n",
+  CLUTTER_NOTE (BACKEND,
+                "Checking features\n"
+                "  GL_VENDOR: %s\n"
+                "  GL_RENDERER: %s\n"
+                "  GL_VERSION: %s\n"
+                "  GL_EXTENSIONS: %s\n",
                 glGetString (GL_VENDOR),
                 glGetString (GL_RENDERER),
                 glGetString (GL_VERSION),
@@ -258,7 +266,7 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
   CLUTTER_NOTE (BACKEND, "GLX Extensions: %s", glx_extensions);
 
   /* First check for explicit disabling or it set elsewhere (eg NVIDIA) */
-  if (getenv("__GL_SYNC_TO_VBLANK") || check_vblank_env ("none"))
+  if (getenv ("__GL_SYNC_TO_VBLANK") || check_vblank_env ("none"))
     {
       CLUTTER_NOTE (BACKEND, "vblank sync: disabled at user request");
     }
@@ -275,7 +283,7 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
        * How well glXGetVideoSyncSGI works with other driver (ATI etc) needs
        * to be investigated. glXGetVideoSyncSGI on ATI at least seems to have
        * no effect.
-      */
+       */
       if (!check_vblank_env ("dri") &&
           cogl_check_extension ("GLX_SGI_swap_control", glx_extensions))
         {
@@ -323,10 +331,11 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
           if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
             CLUTTER_NOTE (BACKEND, "glXGetVideoSyncSGI vblank setup failed");
         }
+
 #ifdef __linux__
       /*
        * DRI is really an extreme fallback -rumoured to work with Via chipsets
-      */
+       */
       if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
         {
           CLUTTER_NOTE (BACKEND, "attempting DRI vblank setup");
@@ -341,33 +350,17 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
           if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
             CLUTTER_NOTE (BACKEND, "DRI vblank setup failed");
         }
-#endif
+#endif /* __linux__ */
+
       if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
-        {
-          CLUTTER_NOTE (BACKEND,
-                        "no use-able vblank mechanism found");
-        }
+        CLUTTER_NOTE (BACKEND, "no use-able vblank mechanism found");
     }
 
-  CLUTTER_NOTE (MISC, "backend features checked");
+  CLUTTER_NOTE (BACKEND, "backend features checked");
 
   return flags;
 }
 
-enum
-{
-  DRAWABLE_TYPE         = 0,
-  RENDER_TYPE           = 2,
-  DOUBLE_BUFFER         = 4,
-  RED_SIZE              = 6,
-  GREEN_SIZE            = 8,
-  BLUE_SIZE             = 10,
-  ALPHA_SIZE            = 12,
-  DEPTH_SIZE            = 14,
-  STENCIL_SIZE          = 16,
-  TRANSPARENT_TYPE      = 18
-};
-
 /* It seems the GLX spec never defined an invalid GLXFBConfig that
  * we could overload as an indication of error, so we have to return
  * an explicit boolean status. */
@@ -377,10 +370,9 @@ _clutter_backend_glx_get_fbconfig (ClutterBackendGLX *backend_glx,
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend_glx);
   GLXFBConfig *configs = NULL;
-  gboolean retval = FALSE;
-  gboolean use_argb = clutter_x11_has_argb_visuals ();
+  gboolean use_argb = clutter_x11_get_use_argb_visual ();
   int n_configs, i;
-  static int attributes[] = {
+  static const int attributes[] = {
     GLX_DRAWABLE_TYPE,    GLX_WINDOW_BIT,
     GLX_RENDER_TYPE,      GLX_RGBA_BIT,
     GLX_DOUBLEBUFFER,     GL_TRUE,
@@ -390,97 +382,77 @@ _clutter_backend_glx_get_fbconfig (ClutterBackendGLX *backend_glx,
     GLX_ALPHA_SIZE,       1,
     GLX_DEPTH_SIZE,       1,
     GLX_STENCIL_SIZE,     1,
-    GLX_TRANSPARENT_TYPE, GLX_NONE,
     None
   };
 
   if (backend_x11->xdpy == None || backend_x11->xscreen == None)
     return FALSE;
 
-  if (backend_glx->found_fbconfig > 0)
+  /* If we don't already have a cached config then try to get one */
+  if (!backend_glx->found_fbconfig)
     {
-      if (use_argb && backend_glx->found_fbconfig == 2)
-        *config = backend_glx->fbconfig_rgba;
-      else
-        *config = backend_glx->fbconfig_rgb;
+      CLUTTER_NOTE (BACKEND,
+                    "Retrieving GL fbconfig, dpy: %p, xscreen; %p (%d)",
+                    backend_x11->xdpy,
+                    backend_x11->xscreen,
+                    backend_x11->xscreen_num);
 
-      return TRUE;
-    }
-
-  if (use_argb)
-    {
-      attributes[ALPHA_SIZE] = 8;
-      attributes[TRANSPARENT_TYPE] = GLX_TRANSPARENT_RGB;
-    }
-
-  CLUTTER_NOTE (BACKEND,
-                "Retrieving GL fbconfig, dpy: %p, xscreen; %p (%d)",
-                backend_x11->xdpy,
-                backend_x11->xscreen,
-                backend_x11->xscreen_num);
-
-  configs = glXChooseFBConfig (backend_x11->xdpy,
-                               backend_x11->xscreen_num,
-                               attributes,
-                               &n_configs);
-  if (!configs)
-    return FALSE;
-
-  if (!use_argb)
-    {
-      *config = configs[0];
-
-      backend_glx->found_fbconfig = 1;
-      backend_glx->fbconfig_rgb = configs[0];
-
-      retval = TRUE;
-
-      goto out;
-    }
-
-  for (i = 0; i < n_configs; i++)
-    {
-      XVisualInfo *vinfo;
-
-      vinfo = glXGetVisualFromFBConfig (backend_x11->xdpy, configs[i]);
-      if (vinfo == None)
-        continue;
-
-      if (vinfo->depth == 32 &&
-          (vinfo->red_mask   == 0xff0000 &&
-           vinfo->green_mask == 0x00ff00 &&
-           vinfo->blue_mask  == 0x0000ff))
+      configs = glXChooseFBConfig (backend_x11->xdpy,
+                                   backend_x11->xscreen_num,
+                                   attributes,
+                                   &n_configs);
+      if (configs)
         {
-          CLUTTER_NOTE (BACKEND, "Found GLX visual ARGB [index:%d]", i);
+          if (use_argb)
+            {
+              for (i = 0; i < n_configs; i++)
+                {
+                  XVisualInfo *vinfo;
 
-          *config = configs[i];
+                  vinfo = glXGetVisualFromFBConfig (backend_x11->xdpy,
+                                                    configs[i]);
+                  if (vinfo == None)
+                    continue;
 
-          backend_glx->found_fbconfig = 2;
-          backend_glx->fbconfig_rgba = configs[i];
+                  if (vinfo->depth == 32 &&
+                      (vinfo->red_mask   == 0xff0000 &&
+                       vinfo->green_mask == 0x00ff00 &&
+                       vinfo->blue_mask  == 0x0000ff))
+                    {
+                      CLUTTER_NOTE (BACKEND,
+                                    "Found GLX visual ARGB [index:%d]", i);
 
-          retval = TRUE;
+                      backend_glx->found_fbconfig = TRUE;
+                      backend_glx->fbconfig = configs[i];
 
-          goto out;
+                      goto out;
+                    }
+                }
+
+              /* If we make it here then we didn't find an RGBA config so
+                 we'll fall back to using an RGB config */
+              CLUTTER_NOTE (BACKEND, "ARGB visual requested, but none found");
+            }
+
+          if (n_configs >= 1)
+            {
+              backend_glx->found_fbconfig = TRUE;
+              backend_glx->fbconfig = configs[0];
+            }
+
+        out:
+          XFree (configs);
         }
     }
 
-  /* XXX - we might add a warning here */
-  if (use_argb && !backend_glx->found_fbconfig != 2)
+  if (backend_glx->found_fbconfig)
     {
-      CLUTTER_NOTE (BACKEND, "ARGB visual requested, but none found");
+      *config = backend_glx->fbconfig;
 
-      *config = configs[0];
-
-      backend_glx->found_fbconfig = 1;
-      backend_glx->fbconfig_rgb = configs[0];
-
-      retval = TRUE;
+      return TRUE;
     }
-
-out:
-  XFree (configs);
-
-  return retval;
+  else
+    return FALSE;
 }
 
 static XVisualInfo *
@@ -500,47 +472,102 @@ clutter_backend_glx_create_context (ClutterBackend  *backend,
                                     GError         **error)
 {
   ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (backend);
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  GLXFBConfig config;
+  gboolean is_direct;
+  Window root_xwin;
+  XSetWindowAttributes attrs;
+  XVisualInfo *xvisinfo;
+  Display *xdisplay;
 
+  if (backend_glx->gl_context != None)
+    return TRUE;
+
+  xdisplay = clutter_x11_get_default_display ();
+  root_xwin = clutter_x11_get_root_window ();
+
+  if (!_clutter_backend_glx_get_fbconfig (backend_glx, &config))
+    {
+      g_set_error (error, CLUTTER_INIT_ERROR,
+                   CLUTTER_INIT_ERROR_BACKEND,
+                   "Unable to find suitable fbconfig for the GLX context");
+      return FALSE;
+    }
+
+  CLUTTER_NOTE (BACKEND, "Creating GLX Context (display: %p)", xdisplay);
+
+  backend_glx->gl_context = glXCreateNewContext (xdisplay,
+                                                 config,
+                                                 GLX_RGBA_TYPE,
+                                                 NULL,
+                                                 True);
   if (backend_glx->gl_context == None)
     {
-      GLXFBConfig config;
-      gboolean is_direct;
+      g_set_error (error, CLUTTER_INIT_ERROR,
+                   CLUTTER_INIT_ERROR_BACKEND,
+                   "Unable to create suitable GL context");
+      return FALSE;
+    }
 
-      if (!_clutter_backend_glx_get_fbconfig (backend_glx, &config))
-        {
-          g_set_error (error, CLUTTER_INIT_ERROR,
-                       CLUTTER_INIT_ERROR_BACKEND,
-                       "Unable to find a suitable GLXFBConfig for "
-                       "the GLX context");
-          return FALSE;
-        }
+  is_direct = glXIsDirect (xdisplay, backend_glx->gl_context);
 
-      CLUTTER_NOTE (GL, "Creating GLX Context (display: %p)",
-                    backend_x11->xdpy);
+  CLUTTER_NOTE (GL, "Setting %s context",
+                is_direct ? "direct"
+                          : "indirect");
+  _cogl_set_indirect_context (!is_direct);
 
-      backend_glx->gl_context =
-        glXCreateNewContext (backend_x11->xdpy,
-                             config,
-                             GLX_RGBA_TYPE,
-                             NULL,
-                             True);
+  /* COGL assumes that there is always a GL context selected; in order
+   * to make sure that a GLX context exists and is made current, we use
+   * a dummy, offscreen override-redirect window to which we can always
+   * fall back if no stage is available
+   *
+   * XXX - we need to do this dance because GLX does not allow creating
+   * a context and querying it for basic information (even the function
+   * pointers) unless it's made current to a real Drawable. it should be
+   * possible to avoid this in future releases of Mesa and X11, but right
+   * now this is the best solution available.
+   */
+  xvisinfo = glXGetVisualFromFBConfig (xdisplay, config);
+  if (xvisinfo == None)
+    {
+      g_set_error (error, CLUTTER_INIT_ERROR,
+                   CLUTTER_INIT_ERROR_BACKEND,
+                   "Unable to retrieve the X11 visual");
+      return FALSE;
+    }
 
-      if (backend_glx->gl_context == None)
-        {
-          g_set_error (error, CLUTTER_INIT_ERROR,
-                       CLUTTER_INIT_ERROR_BACKEND,
-                       "Unable to create suitable GL context");
-          return FALSE;
-        }
+  clutter_x11_trap_x_errors ();
 
-      is_direct = glXIsDirect (backend_x11->xdpy,
-                               backend_glx->gl_context);
+  attrs.override_redirect = True;
+  attrs.colormap = XCreateColormap (xdisplay,
+                                    root_xwin,
+                                    xvisinfo->visual,
+                                    AllocNone);
 
-      CLUTTER_NOTE (GL,
-                    "Setting %s context",
-                    is_direct ? "direct" : "indirect");
-      _cogl_set_indirect_context (!is_direct);
+  backend_glx->dummy_xwin = XCreateWindow (xdisplay, root_xwin,
+                                           -100, -100, 1, 1,
+                                           0,
+                                           xvisinfo->depth,
+                                           CopyFromParent,
+                                           xvisinfo->visual,
+                                           CWOverrideRedirect | CWColormap,
+                                           &attrs);
+
+  CLUTTER_NOTE (BACKEND, "Selecting dummy 0x%x for the GLX context",
+                (unsigned int) backend_glx->dummy_xwin);
+
+  glXMakeContextCurrent (xdisplay,
+                         backend_glx->dummy_xwin,
+                         backend_glx->dummy_xwin,
+                         backend_glx->gl_context);
+
+  XFree (xvisinfo);
+
+  if (clutter_x11_untrap_x_errors ())
+    {
+      g_set_error (error, CLUTTER_INIT_ERROR,
+                   CLUTTER_INIT_ERROR_BACKEND,
+                   "Unable to select the newly created GLX context");
+      return FALSE;
     }
 
   return TRUE;
@@ -575,15 +602,16 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
 
       g_assert (impl != NULL);
 
-      CLUTTER_NOTE (MULTISTAGE, "Setting context for stage of type %s [%p]",
-                    g_type_name (G_OBJECT_TYPE (impl)),
-                    impl);
-
       stage_glx = CLUTTER_STAGE_GLX (impl);
       stage_x11 = CLUTTER_STAGE_X11 (impl);
       backend_glx = CLUTTER_BACKEND_GLX (backend);
       backend_x11 = CLUTTER_BACKEND_X11 (backend);
 
+      CLUTTER_NOTE (BACKEND,
+                    "Setting context for stage of type %s, window: 0x%x",
+                    G_OBJECT_TYPE_NAME (impl),
+                    (unsigned int) stage_x11->xwin);
+
       /* no GL context to set */
       if (backend_glx->gl_context == None)
         return;
@@ -595,17 +623,25 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
        */
       if (stage_x11->xwin == None)
         {
-          CLUTTER_NOTE (MULTISTAGE,
+          CLUTTER_NOTE (BACKEND,
                         "Received a stale stage, clearing all context");
 
-          glXMakeContextCurrent (backend_x11->xdpy, None, None, NULL);
+          if (backend_glx->dummy_xwin != None)
+            {
+              glXMakeContextCurrent (backend_x11->xdpy,
+                                     backend_glx->dummy_xwin,
+                                     backend_glx->dummy_xwin,
+                                     backend_glx->gl_context);
+            }
+          else
+            glXMakeContextCurrent (backend_x11->xdpy, None, None, NULL);
         }
       else
         {
           CLUTTER_NOTE (BACKEND,
                         "MakeContextCurrent dpy: %p, window: 0x%x (%s), context: %p",
                         backend_x11->xdpy,
-                        (int) stage_x11->xwin,
+                        (unsigned int) stage_x11->xwin,
                         stage_x11->is_foreign_xwin ? "foreign" : "native",
                         backend_glx->gl_context);
 
@@ -618,7 +654,7 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
       if (clutter_x11_untrap_x_errors ())
         g_critical ("Unable to make the stage window 0x%x the current "
                     "GLX drawable",
-                    (int) stage_x11->xwin);
+                    (unsigned int) stage_x11->xwin);
     }
 }
 
@@ -689,6 +725,16 @@ clutter_backend_glx_redraw (ClutterBackend *backend,
   ClutterStageGLX *stage_glx;
   ClutterStageX11 *stage_x11;
   ClutterStageWindow *impl;
+  CLUTTER_STATIC_TIMER (painting_timer,
+                        "Redrawing", /* parent */
+                        "Painting actors",
+                        "The time spent painting actors",
+                        0 /* no application private data */);
+  CLUTTER_STATIC_TIMER (swapbuffers_timer,
+                        "Redrawing", /* parent */
+                        "glXSwapBuffers",
+                        "The time spent blocked by glXSwapBuffers",
+                        0 /* no application private data */);
 
   impl = _clutter_stage_get_window (stage);
   if (G_UNLIKELY (impl == NULL))
@@ -703,9 +749,11 @@ clutter_backend_glx_redraw (ClutterBackend *backend,
   stage_x11 = CLUTTER_STAGE_X11 (impl);
   stage_glx = CLUTTER_STAGE_GLX (impl);
 
+  CLUTTER_TIMER_START (_clutter_uprof_context, painting_timer);
   /* this will cause the stage implementation to be painted */
   clutter_actor_paint (CLUTTER_ACTOR (stage));
   cogl_flush ();
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, painting_timer);
 
   if (stage_x11->xwin != None)
     {
@@ -717,7 +765,10 @@ clutter_backend_glx_redraw (ClutterBackend *backend,
       CLUTTER_NOTE (BACKEND, "glXSwapBuffers (display: %p, window: 0x%lx)",
                     backend_x11->xdpy,
                     (unsigned long) stage_x11->xwin);
+
+      CLUTTER_TIMER_START (_clutter_uprof_context, swapbuffers_timer);
       glXSwapBuffers (backend_x11->xdpy, stage_x11->xwin);
+      CLUTTER_TIMER_STOP (_clutter_uprof_context, swapbuffers_timer);
     }
 }
 
